'use strict'

var imp = require('../_js/testImports')
var readFileGo = require('../../index')
var badPath = '/i/do/not/exist'
var path = __dirname + '/readFileGo.txt'

describe('readFileGo', function() {

  var DEFAULTS, sandbox

  before(function() {
    imp.logger.setLevel('silent')
  })

  beforeEach(function() {
    delete require.cache[require.resolve('../../js/normaliseOptions/defaultOptions')]
    DEFAULTS = require('../../js/normaliseOptions/defaultOptions')
    sandbox = imp.sinon.sandbox.create()
    sandbox.spy(imp.logger, 'info')
    sandbox.stub(DEFAULTS, 'logFunc')
  })

  afterEach(function() {
    sandbox.restore()
  })

  describe('success', function() {

    it('success', function() {
      var contents = readFileGo(path)
      imp.expect(contents).to.be.a.string
      imp.expect(contents).to.equal('hello there\nhow are you?\n')
      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
    })

    it('encoding', function() {
      var contents = readFileGo(path, {encoding: null})
      imp.expect(contents).to.be.a('Uint8Array')
      imp.expect(contents.toString()).to.equal('hello there\nhow are you?\n')
      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
    })
  })

  describe('Unknown fs.readFileSync option', function() {

    it('encoding', function() {
      imp.expect(function() {
        readFileGo(path, {encoding: 'invalid'})
      }).to.throw(/Unknown encoding/)
      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
    })

    it('flag', function() {
      imp.expect(function() {
        readFileGo(path, {flag: 'invalid'})
      }).to.throw(/Unknown file open flag/)
      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
    })
  })

  describe('ENOENT file not found', function() {

    it('ENOENT', function() {
      var contents = readFileGo(badPath)
      imp.expect(contents).to.be.undefined
      imp.expect(DEFAULTS.logFunc).to.have.been.calledWithMatch(/ENOENT/)
    })

    it('ENOENT logFunc', function() {
      readFileGo(badPath, {logFunc: imp.logger.info})
      imp.expect(imp.logger.info).to.have.been.calledWithMatch(/ENOENT/)
      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
    })

    it('ENOENT message', function() {
      readFileGo(badPath, {message: ' woteva '})
      imp.expect(DEFAULTS.logFunc).to.have.been.calledWith('woteva' + ' ' + badPath)
    })

    it('ENOENT silent', function() {
      readFileGo(badPath, {silent: true})
      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
    })

    it('ENOENT throw', function() {
      imp.expect(function() {
        readFileGo(badPath, {throw: true})
      }).to.throw(/ENOENT/)
      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
    })

    it('ENOENT throw with message', function() {
      imp.expect(function() {
        readFileGo(badPath, {throw: true, message: ' woteva '})
      }).to.throw('woteva' + ' ' + badPath)
      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
    })
  })

  describe('save', function() {

    it('encoding', function() {

      var contents

      contents = readFileGo(path)
      imp.expect(contents).to.be.a.string
      imp.expect(contents).to.equal('hello there\nhow are you?\n')

      contents = readFileGo(path, {encoding: null, save: true})
      imp.expect(contents).to.be.a('Uint8Array')
      imp.expect(contents.toString()).to.equal('hello there\nhow are you?\n')

      contents = readFileGo(path)
      imp.expect(contents).to.be.a('Uint8Array')
      imp.expect(contents.toString()).to.equal('hello there\nhow are you?\n')

      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
    })

    it('flag', function() {

      readFileGo(path)

      imp.expect(function() {
        readFileGo(path, {flag: 'invalid', save: true})
      }).to.throw(/Unknown file open flag/)

      imp.expect(function() {
        readFileGo(path)
      }).to.throw(/Unknown file open flag/)

      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
    })

    it('logFunc', function() {

      readFileGo(badPath)
      imp.expect(DEFAULTS.logFunc).to.have.been.calledOnce
      imp.expect(DEFAULTS.logFunc.firstCall).to.have.been.calledWithMatch(/ENOENT/)
      imp.expect(DEFAULTS.logFunc).to.not.equal(imp.logger.info)

      readFileGo(badPath, {logFunc: imp.logger.info, save: true})
      imp.expect(DEFAULTS.logFunc).to.have.been.calledOnce
      imp.expect(DEFAULTS.logFunc.firstCall).to.have.been.calledWithMatch(/ENOENT/)
      imp.expect(DEFAULTS.logFunc).to.equal(imp.logger.info)

      readFileGo(badPath)
      imp.expect(DEFAULTS.logFunc).to.have.been.calledTwice
      imp.expect(DEFAULTS.logFunc.secondCall).to.have.been.calledWithMatch(/ENOENT/)
      imp.expect(DEFAULTS.logFunc).to.equal(imp.logger.info)
    })

    it('silent', function() {

      readFileGo(badPath)
      imp.expect(DEFAULTS.logFunc).to.have.been.calledWithMatch(/ENOENT/)
      DEFAULTS.logFunc.reset()

      readFileGo(badPath, {silent: true, save: true})
      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
      DEFAULTS.logFunc.reset()

      readFileGo(badPath)
      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
      DEFAULTS.logFunc.reset()
    })

    it('throw', function() {

      readFileGo(badPath)
      imp.expect(DEFAULTS.logFunc).to.have.been.calledWithMatch(/ENOENT/)
      DEFAULTS.logFunc.reset()

      imp.expect(function() {
        readFileGo(badPath, {throw: true, save: true})
      }).to.throw(/ENOENT/)
      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
      DEFAULTS.logFunc.reset()

      imp.expect(function() {
        readFileGo(badPath)
      }).to.throw(/ENOENT/)
      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
      DEFAULTS.logFunc.reset()
    })
  })

  describe('saveMessage', function() {

    it('message', function() {

      var message = 'woteva'

      readFileGo(badPath)
      imp.expect(DEFAULTS.logFunc).to.have.been.calledWithMatch(/ENOENT/)
      DEFAULTS.logFunc.reset()

      readFileGo(badPath, {message: message, saveMessage: true})
      imp.expect(DEFAULTS.logFunc).to.have.been.calledWith(message + ' ' + badPath)
      DEFAULTS.logFunc.reset()

      readFileGo(badPath)
      imp.expect(DEFAULTS.logFunc).to.have.been.calledWith(message + ' ' + badPath)
      DEFAULTS.logFunc.reset()
    })
  })

  describe('invalid file', function() {

    it('not a string', function() {
      imp.expect(function() {
        readFileGo(9)
      }).to.throw(TypeError, 'cannot read file because file must be a string containing the file path')
      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
    })

    it('undefined', function() {
      imp.expect(function() {
        readFileGo()
      }).to.throw(TypeError, 'cannot read file because file is undefined')
      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
    })

    it('null', function() {
      imp.expect(function() {
        readFileGo(null)
      }).to.throw(TypeError, 'cannot read file because file is null')
      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
    })
  })

  describe('invalid options', function() {

    it('options is not an object', function() {
      readFileGo(badPath, 9)
      imp.expect(DEFAULTS.logFunc).to.have.been.calledWithMatch(/ENOENT/)
    })

    it('encoding is not a string', function() {
      var contents = readFileGo(path, {encoding: 9})
      imp.expect(contents).to.equal('hello there\nhow are you?\n')
      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
    })

    it('flag is not a string', function() {
      var contents = readFileGo(path, {flag: 9})
      imp.expect(contents).to.equal('hello there\nhow are you?\n')
      imp.expect(DEFAULTS.logFunc).to.have.not.been.called
    })

    it('logFunc is not a function', function() {
      readFileGo(badPath, {logFunc: 9})
      imp.expect(DEFAULTS.logFunc).to.have.been.calledWithMatch(/ENOENT/)
    })

    it('message is not a string', function() {
      readFileGo(badPath, {message: 9})
      imp.expect(DEFAULTS.logFunc).to.have.been.calledWithMatch(/ENOENT/)
    })
  })
})
